module rgb2gray
(
    input        clk, rstb,
    input        i_hav, i_vav,
    input  [7:0] i_r, i_g, i_b,
    output       o_hav, o_vav,
    output [7:0] o_y
);

// Clocking input data
reg       hav, vav;
reg [7:0] r, g, b;

always @(posedge clk or negedge rstb) begin
   if (~rstb) begin
      hav <= 0;
      vav <= 0;
      r   <= 0;
      g   <= 0;
      b   <= 0;
   end
   else begin
      hav <= i_hav;
      vav <= i_vav;
      r   <= (i_hav&i_vav) ? i_r : 0;
      g   <= (i_hav&i_vav) ? i_g : 0;
      b   <= (i_hav&i_vav) ? i_b : 0;
   end
end

// y = (77r + 150g + 29b)>>8
reg [15:0] y_r, y_g, y_b;
reg        hav_t1, vav_t1;

always @(posedge clk or negedge rstb) begin
   if (~rstb) begin
      y_r    <= 0;
      y_g    <= 0;
      y_b    <= 0;
      hav_t1 <= 0;
      vav_t1 <= 0;
   end
   else begin
      y_r    <= r * 8'd77;
      y_g    <= g * 8'd150;
      y_b    <= b * 8'd29;
      hav_t1 <= hav;
      vav_t1 <= vav;
   end
end

reg [15:0] y;
reg        hav_t2, vav_t2;

always @(posedge clk or negedge rstb) begin
   if (~rstb) begin
      y      <= 0;
      hav_t2 <= 0;
      vav_t2 <= 0;
   end
   else begin
      y      <= y_r + y_g + y_b;
      hav_t2 <= hav_t1;
      vav_t2 <= vav_t1;
   end
end

// filtering
reg [7:0] y_d1, y_d2;
reg       hav_t3, vav_t3;

always @(posedge clk or negedge rstb) begin
   if (~rstb) begin
      y_d1   <= 0;
      y_d2   <= 0;
      hav_t3 <= 0;
      vav_t3 <= 0;
   end
   else begin
      y_d1   <= y[15:8];
      y_d2   <= y_d1;
      hav_t3 <= hav_t2;
      vav_t3 <= vav_t2;
   end
end

reg [15:0] f, f1, f2, f3;
reg        hav_t4, vav_t4, hav_t5, vav_t5;

always @(posedge clk or negedge rstb) begin
   if (~rstb) begin
      f1 <= 0;
      f2 <= 0;
      f3 <= 0;
      f  <= 0;
      hav_t4 <= 0;
      vav_t4 <= 0;
      hav_t5 <= 0;
      vav_t5 <= 0;
   end
   else begin
      f1 <= y[15:8] * 8'd85;
      f2 <= y_d1 * 8'd85;
      f3 <= y_d2 * 8'd85;
      f  <= f1 + f2 + f3;
      hav_t4 <= hav_t3;
      vav_t4 <= vav_t3;
      hav_t5 <= hav_t4;
      vav_t5 <= vav_t4;
   end
end

// histogram
reg [18:0] hist[0:255];

always @(posedge clk or negedge rstb) begin
   if (~rstb) begin
      hist[0] = 0;
      hist[1] = 0;
      hist[2] = 0;
      hist[3] = 0;
      hist[4] = 0;
      hist[5] = 0;
      hist[6] = 0;
      hist[7] = 0;
      hist[8] = 0;
      hist[9] = 0;
      hist[10] = 0;
      hist[11] = 0;
      hist[12] = 0;
      hist[13] = 0;
      hist[14] = 0;
      hist[15] = 0;
      hist[16] = 0;
      hist[17] = 0;
      hist[18] = 0;
      hist[19] = 0;
      hist[20] = 0;
      hist[21] = 0;
      hist[22] = 0;
      hist[23] = 0;
      hist[24] = 0;
      hist[25] = 0;
      hist[26] = 0;
      hist[27] = 0;
      hist[28] = 0;
      hist[29] = 0;
      hist[30] = 0;
      hist[31] = 0;
      hist[32] = 0;
      hist[33] = 0;
      hist[34] = 0;
      hist[35] = 0;
      hist[36] = 0;
      hist[37] = 0;
      hist[38] = 0;
      hist[39] = 0;
      hist[40] = 0;
      hist[41] = 0;
      hist[42] = 0;
      hist[43] = 0;
      hist[44] = 0;
      hist[45] = 0;
      hist[46] = 0;
      hist[47] = 0;
      hist[48] = 0;
      hist[49] = 0;
      hist[50] = 0;
      hist[51] = 0;
      hist[52] = 0;
      hist[53] = 0;
      hist[54] = 0;
      hist[55] = 0;
      hist[56] = 0;
      hist[57] = 0;
      hist[58] = 0;
      hist[59] = 0;
      hist[60] = 0;
      hist[61] = 0;
      hist[62] = 0;
      hist[63] = 0;
      hist[64] = 0;
      hist[65] = 0;
      hist[66] = 0;
      hist[67] = 0;
      hist[68] = 0;
      hist[69] = 0;
      hist[70] = 0;
      hist[71] = 0;
      hist[72] = 0;
      hist[73] = 0;
      hist[74] = 0;
      hist[75] = 0;
      hist[76] = 0;
      hist[77] = 0;
      hist[78] = 0;
      hist[79] = 0;
      hist[80] = 0;
      hist[81] = 0;
      hist[82] = 0;
      hist[83] = 0;
      hist[84] = 0;
      hist[85] = 0;
      hist[86] = 0;
      hist[87] = 0;
      hist[88] = 0;
      hist[89] = 0;
      hist[90] = 0;
      hist[91] = 0;
      hist[92] = 0;
      hist[93] = 0;
      hist[94] = 0;
      hist[95] = 0;
      hist[96] = 0;
      hist[97] = 0;
      hist[98] = 0;
      hist[99] = 0;
      hist[100] = 0;
      hist[101] = 0;
      hist[102] = 0;
      hist[103] = 0;
      hist[104] = 0;
      hist[105] = 0;
      hist[106] = 0;
      hist[107] = 0;
      hist[108] = 0;
      hist[109] = 0;
      hist[110] = 0;
      hist[111] = 0;
      hist[112] = 0;
      hist[113] = 0;
      hist[114] = 0;
      hist[115] = 0;
      hist[116] = 0;
      hist[117] = 0;
      hist[118] = 0;
      hist[119] = 0;
      hist[120] = 0;
      hist[121] = 0;
      hist[122] = 0;
      hist[123] = 0;
      hist[124] = 0;
      hist[125] = 0;
      hist[126] = 0;
      hist[127] = 0;
      hist[128] = 0;
      hist[129] = 0;
      hist[130] = 0;
      hist[131] = 0;
      hist[132] = 0;
      hist[133] = 0;
      hist[134] = 0;
      hist[135] = 0;
      hist[136] = 0;
      hist[137] = 0;
      hist[138] = 0;
      hist[139] = 0;
      hist[140] = 0;
      hist[141] = 0;
      hist[142] = 0;
      hist[143] = 0;
      hist[144] = 0;
      hist[145] = 0;
      hist[146] = 0;
      hist[147] = 0;
      hist[148] = 0;
      hist[149] = 0;
      hist[150] = 0;
      hist[151] = 0;
      hist[152] = 0;
      hist[153] = 0;
      hist[154] = 0;
      hist[155] = 0;
      hist[156] = 0;
      hist[157] = 0;
      hist[158] = 0;
      hist[159] = 0;
      hist[160] = 0;
      hist[161] = 0;
      hist[162] = 0;
      hist[163] = 0;
      hist[164] = 0;
      hist[165] = 0;
      hist[166] = 0;
      hist[167] = 0;
      hist[168] = 0;
      hist[169] = 0;
      hist[170] = 0;
      hist[171] = 0;
      hist[172] = 0;
      hist[173] = 0;
      hist[174] = 0;
      hist[175] = 0;
      hist[176] = 0;
      hist[177] = 0;
      hist[178] = 0;
      hist[179] = 0;
      hist[180] = 0;
      hist[181] = 0;
      hist[182] = 0;
      hist[183] = 0;
      hist[184] = 0;
      hist[185] = 0;
      hist[186] = 0;
      hist[187] = 0;
      hist[188] = 0;
      hist[189] = 0;
      hist[190] = 0;
      hist[191] = 0;
      hist[192] = 0;
      hist[193] = 0;
      hist[194] = 0;
      hist[195] = 0;
      hist[196] = 0;
      hist[197] = 0;
      hist[198] = 0;
      hist[199] = 0;
      hist[200] = 0;
      hist[201] = 0;
      hist[202] = 0;
      hist[203] = 0;
      hist[204] = 0;
      hist[205] = 0;
      hist[206] = 0;
      hist[207] = 0;
      hist[208] = 0;
      hist[209] = 0;
      hist[210] = 0;
      hist[211] = 0;
      hist[212] = 0;
      hist[213] = 0;
      hist[214] = 0;
      hist[215] = 0;
      hist[216] = 0;
      hist[217] = 0;
      hist[218] = 0;
      hist[219] = 0;
      hist[220] = 0;
      hist[221] = 0;
      hist[222] = 0;
      hist[223] = 0;
      hist[224] = 0;
      hist[225] = 0;
      hist[226] = 0;
      hist[227] = 0;
      hist[228] = 0;
      hist[229] = 0;
      hist[230] = 0;
      hist[231] = 0;
      hist[232] = 0;
      hist[233] = 0;
      hist[234] = 0;
      hist[235] = 0;
      hist[236] = 0;
      hist[237] = 0;
      hist[238] = 0;
      hist[239] = 0;
      hist[240] = 0;
      hist[241] = 0;
      hist[242] = 0;
      hist[243] = 0;
      hist[244] = 0;
      hist[245] = 0;
      hist[246] = 0;
      hist[247] = 0;
      hist[248] = 0;
      hist[249] = 0;
      hist[250] = 0;
      hist[251] = 0;
      hist[252] = 0;
      hist[253] = 0;
      hist[254] = 0;
      hist[255] = 0;
   end
   else begin
      hist[f[15:8]] <= (hav_t5&vav_t5) ? hist[f[15:8]]+1 : hist[f[15:8]];
   end
end

// output
assign o_y   = f[15:8];
assign o_hav = hav_t5;
assign o_vav = vav_t5;

endmodule