`timescale 1 ns/1 ps

module testbench;

localparam WIDTH  = 512;
localparam HEIGHT = 512;

reg        clk;
reg        rstb;
reg  [7:0] i_r;
reg  [7:0] i_g;
reg  [7:0] i_b;

wire [7:0] o_y;
wire       o_hav;
wire       o_vav;

wire       hav;
wire       vav;
reg        hav_ss;
reg        vav_ss;

reg  [7:0] r_data[0:HEIGHT*WIDTH-1];
reg  [7:0] g_data[0:HEIGHT*WIDTH-1];
reg  [7:0] b_data[0:HEIGHT*WIDTH-1];

integer    i;
integer    result_y_id, result_hist_id;

// Unit instantiation
hav_vav_gen 
#(
   .WIDTH  (WIDTH-1 ),
   .HEIGHT (HEIGHT-1),
   .H_BLANK(51      ),
   .V_BLANK(51      )
) sync_gen (
   .clk    (clk     ),
   .rstb   (rstb    ),
   .hav    (hav     ),
   .vav    (vav     )
);

rgb2gray uut
(
   .clk  (clk   ),
   .rstb (rstb  ),
   .i_hav(hav_ss),
   .i_vav(vav_ss),
   .i_r  (i_r   ),
   .i_g  (i_g   ),
   .i_b  (i_b   ),
   .o_y  (o_y   ),
   .o_hav(o_hav ),
   .o_vav(o_vav )
);

// Clock generation
always #10 clk = ~clk;

// Other stimuli
initial begin
   result_y_id  = $fopen("../py/out_y.txt", "w");
   result_hist_id  = $fopen("../py/hist.txt", "w");
   rstb = 0;
   clk  = 0;
   i    = 0;
   @(negedge clk) rstb = 1;
   $readmemh("../py/in_img_r.txt", r_data);
   $readmemh("../py/in_img_g.txt", g_data);
   $readmemh("../py/in_img_b.txt", b_data);
end

always @(posedge clk or negedge rstb)
   if (!rstb) begin
      hav_ss <= 1'b0;
      vav_ss <= 1'b0;
   end
   else begin
      hav_ss <= hav;
      vav_ss <= vav;
   end

always @(posedge clk or negedge rstb)
   if (!rstb) begin
      i_r <= 8'd0;
      i_g <= 8'd0;
      i_b <= 8'd0;
   end
   else begin
      if (o_hav & o_vav) begin
         $fwrite(result_y_id, "%d\n", o_y);
      end

      if (hav & vav) begin
         i_r <= r_data[i];
         i_g <= g_data[i];
         i_b <= b_data[i];
         i   <= i + 1;
      end

      if (i == HEIGHT*WIDTH+1) begin
         for (i=0; i<256; i=i+1) begin
            $fwrite(result_hist_id, "%d\n", uut.hist[i]);
         end
         $fclose(result_y_id);
         $fclose(result_hist_id);
         $stop;
      end
   end

endmodule
